/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 * 
 * 21.11.2008 - [JR] - creation
 * 26.11.2008 - [JR] - BorderLayout as default layout
 */
package javax.rad.application.genui;

import javax.rad.application.IConnectable;
import javax.rad.application.IWorkScreenApplication;
import javax.rad.genui.layout.UIBorderLayout;
import javax.rad.remote.AbstractConnection;

/**
 * The <code>RemoteWorkScreen</code> extends the {@link WorkScreen} with
 * an {@link AbstractConnection}.
 * 
 * @author Ren Jahn
 */
public class RemoteWorkScreen extends WorkScreen 
                              implements IConnectable
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Class members
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/** the connection of the work-screen. */
	private AbstractConnection connection;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Initialization
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
	 * Creates a new instance of <code>RemoteWorkScreen</code> for a parent
	 * application.
	 * 
	 * @param pApplication the parent application
	 */
	public RemoteWorkScreen(IWorkScreenApplication pApplication)
	{
		this(pApplication, null);
	}
	
	/**
	 * Creates a new instance of <code>RemoteWorkScreen</code> with a desired
	 * launcher and connection.
	 * 
	 * @param pApplication the parent application
	 * @param pConnection the connection
	 */
	public RemoteWorkScreen(IWorkScreenApplication pApplication, AbstractConnection pConnection)
	{
		super(pApplication);
		
		connection = pConnection;
		
		setLayout(new UIBorderLayout());
	}
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Interface implementation
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	/**
	 * {@inheritDoc}
	 */
	public AbstractConnection getConnection()
	{
		return connection;
	}

	/**
	 * {@inheritDoc}
	 */
	public void setConnection(AbstractConnection pConnection)
	{
		connection = pConnection;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public boolean isConnected()
	{
		AbstractConnection con = getConnection();
		
		return con != null && con.isOpen();
	}

	/**
	 * {@inheritDoc}
	 */
	public RemoteWorkScreenApplication getApplication()
	{
		return (RemoteWorkScreenApplication)application;
	}
	
}	// RemoteWorkScreen
