/*
 * Copyright 2009 SIB Visions GmbH
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 *
 *
 * History
 *
 * 20.04.2008 - [HM] - creation
 */
package javax.rad.ui.event;

/**
 * Platform and technology independent key listener definition.
 * It is designed for use with AWT, Swing, SWT, JSP, JSF, ... .
 * 
 * @author Martin Handsteiner
 * @see    java.awt.event.KeyListener
 */
public interface IKeyListener
{
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Method definitions
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	/**
     * Invoked when a key has been typed.
     * See the class description for {@link java.awt.event.KeyEvent} for a definition of 
     * a key typed event.
     * 
     * @param pKeyEvent the key event.
     */
    public void keyTyped(UIKeyEvent pKeyEvent);

    /**
     * Invoked when a key has been pressed. 
     * See the class description for {@link java.awt.event.KeyEvent} for a definition of 
     * a key pressed event.
     * 
     * @param pKeyEvent the key event.
     */
    public void keyPressed(UIKeyEvent pKeyEvent);

    /**
     * Invoked when a key has been released.
     * See the class description for {@link java.awt.event.KeyEvent} for a definition of 
     * a key released event.
     * @param pKeyEvent the key event.
     */
    public void keyReleased(UIKeyEvent pKeyEvent);

}	// IKeyListener
